package ip_2.lab1;

import ip_2.lab1.items.model.ItemEntity;
import ip_2.lab1.items.service.ItemService;
import ip_2.lab1.reviews.model.ReviewEntity;
import ip_2.lab1.reviews.service.ReviewService;
import ip_2.lab1.types.model.TypeEntity;
import ip_2.lab1.types.service.TypeService;
import ip_2.lab1.users.model.UserEntity;
import ip_2.lab1.users.service.UserService;
import org.h2.engine.User;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

import java.util.Objects;

@SpringBootApplication
public class Lab1Application implements CommandLineRunner {
	private final Logger log = LoggerFactory.getLogger(Lab1Application.class);

	private final TypeService typeService;
	private final ItemService itemService;
	private final UserService userService;
	private final ReviewService reviewService;

	public Lab1Application(TypeService typeService, ItemService itemService, UserService userService, ReviewService reviewService) {
		this.typeService = typeService;
		this.itemService = itemService;
		this.userService = userService;
		this.reviewService = reviewService;
	}

	public static void main(String[] args) {
		SpringApplication.run(Lab1Application.class, args);
	}

	@Override
	public void run(String... args) throws Exception {
		if (args.length > 0 && Objects.equals("--populate", args[0])) {
			log.info("Create default types values");
			final var type1 = typeService.create(new TypeEntity("Асфальт", 2000));
			final var type2 = typeService.create(new TypeEntity("Грунт", 3000));
			final var type3 = typeService.create(new TypeEntity("Асфальт + грунт", 3500));

			log.info("Create items");
			final var item1 = itemService.create(new ItemEntity(type1, 1700, 5));
			final var item2 = itemService.create(new ItemEntity(type2, 2700, 5));
			final var item3 = itemService.create(new ItemEntity(type3, 3200, 5));

			log.info("Create users");
			final var user1 = userService.create(new UserEntity("Artyom", "1234"));
			final var user2 = userService.create(new UserEntity("Niyaz", "1234"));
			final var user3 = userService.create(new UserEntity("Rostik", "1234"));
			final var user4 = userService.create(new UserEntity("Artur", "1234"));
			final var user5 = userService.create(new UserEntity("Andrey", "1234"));
			final var user6 = userService.create(new UserEntity("Alionchik", "1234"));


			log.info("Create reviews");
			final var review1 = reviewService.create(new ReviewEntity(user1, "Кайфарики"));
			final var review2 = reviewService.create(new ReviewEntity(user2, "Кайфарики кайфецкие"));
			final var review3 = reviewService.create(new ReviewEntity(user3, "Кайфарики турецкие"));
			final var review4 = reviewService.create(new ReviewEntity(user3, "Кайфарики масдавские"));
			final var review5 = reviewService.create(new ReviewEntity(user4, "Кайфарики минские"));
			final var review7 = reviewService.create(new ReviewEntity(user4, "Кайфарики грузинские"));
			final var review8 = reviewService.create(new ReviewEntity(user5, "Кайфарики японские"));
			final var review9 = reviewService.create(new ReviewEntity(user6, "Кайфарики македонские"));


			log.info("Creation finished");
		}
	}
}
