package ip_2.lab1.items.api;

import ip_2.lab1.core.configuration.Constants;
import ip_2.lab1.items.model.ItemEntity;
import ip_2.lab1.items.model.ItemGrouped;
import ip_2.lab1.items.service.ItemService;
import ip_2.lab1.types.service.TypeService;
import jakarta.validation.Valid;
import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.*;
import org.yaml.snakeyaml.nodes.CollectionNode;

import java.util.List;

@RestController
@RequestMapping(Constants.API_URL + "/item")
public class ItemController {
    private final TypeService typeService;
    private final ItemService itemService;
    private final ModelMapper modelMapper;

    public ItemController(TypeService typeService, ItemService itemService, ModelMapper modelMapper) {
        this.itemService = itemService;
        this.typeService = typeService;
        this.modelMapper = modelMapper;
    }

    private ItemDto toDto(ItemEntity entity) {
        final ItemDto dto = modelMapper.map(entity, ItemDto.class);
        double ben = (entity.getType().getBasePrice() - entity.getPrice()) * entity.getCount();
        dto.setBenefit(ben);
        return dto;
    }

    private ItemEntity toEntity(ItemDto dto) {
        final ItemEntity entity = modelMapper.map(dto, ItemEntity.class);
        entity.setType(typeService.get(dto.getTypeId()));
        return entity;
    }
    private ItemGroupedDto toGroupedDto(ItemGrouped entity) {
        return modelMapper.map(entity, ItemGroupedDto.class);
    }


    @GetMapping
    public List<ItemDto> getAll(@RequestParam(name = "typeId", defaultValue = "0") Long typeId) {
        return itemService.getAll(typeId).stream().map(this::toDto).toList();
    }

    @GetMapping("/{id}")
    public ItemDto get(@PathVariable(name = "id") Long id) {
        return toDto(itemService.get(id));
    }

    @PostMapping
    public ItemDto create(
            @RequestBody ItemDto dto) {
        return toDto(itemService.create( toEntity(dto)));
    }

    @PutMapping("/{id}")
    public ItemDto update(@PathVariable(name = "id") Long id, @RequestBody ItemDto dto) {
        return toDto(itemService.update(id, toEntity(dto)));
    }

    @DeleteMapping("/{id}")
    public ItemDto delete(@PathVariable(name = "id") Long id) {
        return toDto(itemService.delete(id));
    }

    @GetMapping("/total")
    public List<ItemGroupedDto> getTotalPrice() {
        return itemService.getTotal().stream()
                .map(this::toGroupedDto)
                .toList();
    }
}
