package ip_2.lab1.items.service;

import ip_2.lab1.core.error.NotFoundException;
import ip_2.lab1.items.model.ItemEntity;
import ip_2.lab1.items.model.ItemGrouped;
import ip_2.lab1.items.repository.ItemRepository;
import ip_2.lab1.types.model.TypeEntity;
import ip_2.lab1.types.service.TypeService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.StreamSupport;

@Service
public class ItemService {
    private final ItemRepository repository;

    public ItemService(ItemRepository repository) {
        this.repository = repository;
    }

    @Transactional(readOnly = true)
    public List<ItemEntity> getAll(long typeId) {
        if (typeId <= 0L) {
            return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
        } else {
            return repository.findByTypeId(typeId);
        }
    }

    @Transactional(readOnly = true)
    public ItemEntity get(long id) {
        return repository.findById(id)
                .orElseThrow(() -> new NotFoundException(ItemEntity.class, id));
    }

    @Transactional
    public ItemEntity create(ItemEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        return repository.save(entity);
    }

    @Transactional
    public ItemEntity update(long id, ItemEntity entity) {
        final ItemEntity existsEntity = get(id);
        existsEntity.setType(entity.getType());
        existsEntity.setPrice(entity.getPrice());
        existsEntity.setCount(entity.getCount());
        return repository.save(existsEntity);
    }

    @Transactional
    public ItemEntity delete(long id) {
        final ItemEntity existsEntity = get(id);
        repository.delete(existsEntity);
        return existsEntity;
    }

    @Transactional(readOnly = true)
    public List<ItemGrouped> getTotal() {
        return repository.getItemsTotalByType();
    }
}
