package ip_2.lab1.reviews.api;

import ip_2.lab1.core.api.PageDto;
import ip_2.lab1.core.api.PageDtoMapper;
import ip_2.lab1.core.configuration.Constants;
import ip_2.lab1.items.api.ItemGroupedDto;
import ip_2.lab1.items.model.ItemGrouped;
import ip_2.lab1.reviews.model.ReviewEntity;
import ip_2.lab1.reviews.model.ReviewGrouped;
import ip_2.lab1.reviews.repository.ReviewRepository;
import ip_2.lab1.reviews.service.ReviewService;
import ip_2.lab1.users.service.UserService;
import jakarta.validation.constraints.Max;
import jakarta.validation.constraints.Min;
import org.modelmapper.ModelMapper;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.jaxb.SpringDataJaxb;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.stream.StreamSupport;

@RestController
@RequestMapping(Constants.API_URL + "/review")
public class ReviewController {
    private final UserService userService;
    private final ReviewService reviewService;
    private final ReviewRepository reviewRepository;
    private final ModelMapper modelMapper;

    public ReviewController(UserService userService, ReviewService reviewService, ReviewRepository reviewRepository, ModelMapper modelMapper) {
        this.userService = userService;
        this.reviewService = reviewService;
        this.reviewRepository = reviewRepository;
        this.modelMapper = modelMapper;
    }

    private ReviewDto toDto(ReviewEntity entity) {
        return modelMapper.map(entity, ReviewDto.class);
    }
    private ReviewEntity toEntity(ReviewDto dto) {
        final ReviewEntity entity = modelMapper.map(dto, ReviewEntity.class);
        entity.setUser(userService.getForReviewCreate(dto.getUserId()));
        return entity;
    }
    private ReviewGroupedDto toGroupedDto(ReviewGrouped entity) {
        return modelMapper.map(entity, ReviewGroupedDto.class);
    }

    @GetMapping
    public PageDto<ReviewDto> getAll(@RequestParam(name = "userId", defaultValue = "0") Long userId,
                          @RequestParam(name = "page", defaultValue = "0") Integer page,
                          @RequestParam(name = "size", defaultValue = "20") Integer size) {
        return PageDtoMapper.toDto(reviewService.getAll(userId, page, size), this::toDto);
    }

    @GetMapping("/{id}")
    public ReviewDto get(@PathVariable(name = "id") Long id) {
        return toDto(reviewService.get(id));
    }

    @PostMapping
    public ReviewDto create(@RequestBody ReviewDto dto) {
        return toDto(reviewService.create(toEntity(dto)));
    }

    @PutMapping("/{id}")
    public ReviewDto update(@PathVariable(name = "id") Long id, @RequestBody ReviewDto dto) {
        return toDto(reviewService.update(id, toEntity(dto)));
    }

    @DeleteMapping("/{id}")
    public ReviewDto delete(@PathVariable(name = "id") Long id) {
        return toDto(reviewService.delete(id));
    }

    @GetMapping("/count")
    public List<ReviewGroupedDto> getReviewCount() {
        return reviewService.getCount(0, 5).stream().map(this::toGroupedDto).toList();
    }
}
