package ip_2.lab1.reviews.service;

import ip_2.lab1.core.error.NotFoundException;
import ip_2.lab1.items.model.ItemGrouped;
import ip_2.lab1.reviews.model.ReviewEntity;
import ip_2.lab1.reviews.model.ReviewGrouped;
import ip_2.lab1.reviews.repository.ReviewRepository;
import ip_2.lab1.types.model.TypeEntity;
import ip_2.lab1.types.repository.TypeRepository;
import ip_2.lab1.users.model.UserEntity;
import ip_2.lab1.users.service.UserService;
import jakarta.validation.constraints.Max;
import jakarta.validation.constraints.Min;
import org.antlr.v4.runtime.atn.SemanticContext;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;
import java.util.Optional;
import java.util.stream.StreamSupport;

@Service
public class ReviewService {
    private final ReviewRepository repository;
    private final UserService userService;

    public ReviewService(ReviewRepository repository, UserService userService) {
        this.repository = repository;
        this.userService = userService;
    }

    @Transactional(readOnly = true)
    public List<ReviewEntity> getAll(long userId) {
        if (userId <= 0L)
            return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
        userService.get(userId);
        return repository.findByUserId(userId);
    }

    @Transactional(readOnly = true)
    public Page<ReviewEntity> getAll(long userId, Integer offset, Integer limit) {
        if (userId <= 0L)
            return repository.findAll(PageRequest.of(offset, limit));
        userService.get(userId);
        return repository.findByUserId(userId, PageRequest.of(offset, limit));
    }

    @Transactional(readOnly = true)
    public ReviewEntity get(Long id) {
        return repository.findById(id).orElseThrow(() -> new NotFoundException(ReviewEntity.class, id));
    }

    @Transactional
    public ReviewEntity create(ReviewEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        final UserEntity existUser = userService.get(entity.getUser().getId());
        entity.setUser(existUser);
        return repository.save(entity);
    }

    @Transactional
    public ReviewEntity update(long id, ReviewEntity entity) {
        final ReviewEntity existsEntity = get(id);
        userService.get(entity.getUser().getId());
        existsEntity.setUser(entity.getUser());
        existsEntity.setText(entity.getText());
        return repository.save(existsEntity);
    }

    @Transactional
    public ReviewEntity delete(long id) {
        final ReviewEntity existsEntity = get(id);
        repository.delete(existsEntity);
        return existsEntity;
    }

    @Transactional(readOnly = true)
    public Page<ReviewGrouped> getCount(int offset, int limit) {
        return repository.getReviewCountByUsers(PageRequest.of(offset, limit));
    }
}
