package ip_2.lab1.types.api;

import ip_2.lab1.core.configuration.Constants;
import ip_2.lab1.types.model.TypeEntity;
import ip_2.lab1.types.service.TypeService;
import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping(Constants.API_URL + "/type")
public class TypeController {
    private final TypeService typeService;
    private final ModelMapper modelMapper;

    public TypeController(TypeService typeService, ModelMapper modelMapper) {
        this.typeService = typeService;
        this.modelMapper = modelMapper;
    }

    private TypeDto toDto(TypeEntity entity) {
        return modelMapper.map(entity, TypeDto.class);
    }
    private TypeEntity toEntity(TypeDto dto) {
        return modelMapper.map(dto, TypeEntity.class);
    }

    @GetMapping
    public List<TypeDto> getAll() {
        return typeService.getAll().stream().map(this::toDto).toList();
    }

    @GetMapping("/{id}")
    public TypeDto get(@PathVariable(name = "id") Long id) {
        return toDto(typeService.get(id));
    }

    @PostMapping
    public TypeDto create(@RequestBody TypeDto dto) {
        return toDto(typeService.create(toEntity(dto)));
    }

    @PutMapping("/{id}")
    public TypeDto update(@PathVariable(name = "id") Long id, @RequestBody TypeDto dto) {
        return toDto(typeService.update(id, toEntity(dto)));
    }

    @DeleteMapping("/{id}")
    public TypeDto delete(@PathVariable(name = "id") Long id) {
        return toDto(typeService.delete(id));
    }
}
