package ip_2.lab1.types.service;

import ip_2.lab1.core.error.NotFoundException;
import ip_2.lab1.items.repository.ItemRepository;
import ip_2.lab1.types.model.TypeEntity;
import ip_2.lab1.types.repository.TypeRepository;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.lang.reflect.Type;
import java.util.List;
import java.util.Optional;
import java.util.stream.StreamSupport;

@Service
public class TypeService {
    private final TypeRepository repository;
    private final ItemRepository itemRepository;

    public TypeService(TypeRepository repository, ItemRepository itemRepository) {
        this.repository = repository;
        this.itemRepository = itemRepository;
    }

    private void checkName(String name) {
        if (repository.findByNameIgnoreCase(name).isPresent()) {
            throw new IllegalArgumentException(
                    String.format("Type with name %s is already exists", name));
        }
    }

    @Transactional(readOnly = true)
    public List<TypeEntity> getAll() {
        return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
    }

    @Transactional(readOnly = true)
    public TypeEntity get(Long id) {
        return repository.findById(id)
                .orElseThrow(() -> new NotFoundException(TypeEntity.class, id));    }

    @Transactional
    public TypeEntity create(TypeEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        checkName(entity.getName());
        return repository.save(entity);    }

    @Transactional
    public TypeEntity update(Long id, TypeEntity entity) {
        final TypeEntity existsEntity = get(id);
        existsEntity.setName(entity.getName());
        existsEntity.setBasePrice(entity.getBasePrice());
        return repository.save(existsEntity);
    }

    public TypeEntity delete(Long id) {
        final TypeEntity existsEntity = get(id);
        itemRepository.deleteAllByTypeId(id);
        repository.delete(existsEntity);
        return existsEntity;
    }
}
