package ip_2.lab1.users.api;

import ip_2.lab1.core.configuration.Constants;
import ip_2.lab1.users.model.UserEntity;
import ip_2.lab1.users.service.UserService;
import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping(Constants.API_URL + "/user")
public class UserController {
    private final UserService userService;
    private final ModelMapper modelMapper;

    public UserController(UserService userService, ModelMapper modelMapper){
        this.userService = userService;
        this.modelMapper = modelMapper;
    }

    private UserDto toDto(UserEntity entity){ return modelMapper.map(entity, UserDto.class);}
    private UserEntity toEntity(UserDto dto){ return modelMapper.map(dto, UserEntity.class);}

    @GetMapping
    public List<UserDto> getAll(){ return userService.getAll().stream().map(this::toDto).toList();}

    @GetMapping("/{id}")
    public UserDto get(@PathVariable(name = "id") Long id){ return toDto(userService.get(id));}

    @PostMapping()
    public UserDto create(@RequestBody UserDto dto){ return toDto(userService.create(toEntity(dto)));}

    @PutMapping("/{id}")
    public UserDto update(@PathVariable(name = "id") Long id, @RequestBody UserDto dto){
        return toDto(userService.update(id, toEntity(dto)));
    }

    @PutMapping("/updatePassword/{id}")
    public UserDto updatePassword(@PathVariable(name = "id") Long id, @RequestParam String password){
        return toDto(userService.updatePassword(id, password));
    }

    @DeleteMapping("/{id}")
    public UserDto delete(@PathVariable(name = "id") Long id){
        return toDto(userService.delete(id));
    }
}
