package ip_2.lab1.users.model;

import ip_2.lab1.core.model.BaseEntity;
import ip_2.lab1.reviews.model.ReviewEntity;
import ip_2.lab1.types.model.TypeEntity;
import jakarta.persistence.*;
import org.h2.engine.User;

import java.util.HashSet;
import java.util.Objects;
import java.util.Set;

@Entity
@Table(name = "users")
public class UserEntity extends BaseEntity {
    @OneToMany(mappedBy = "user", cascade = CascadeType.ALL)
    @OrderBy("id ASC")
    private final Set<ReviewEntity> reviews = new HashSet<>();
    @Column(nullable = false, unique = true, length = 30)
    private String login;
    @Column(nullable = false, length = 30)
    private String password;

    public UserEntity() {
    }

    public UserEntity (String login, String password){
        this.login = login;
        this.password = password;
    }

    public String getLogin(){ return login;}
    public void setLogin(String login) {
        this.login = login;
    }
    public String getPassword() { return password; }
    public void setPassword(String password) {
        this.password = password;
    }

    public Set<ReviewEntity> getReviews() {
        return reviews;
    }

    public void addReview(ReviewEntity review) {
        if (review.getUser() != this) {
            review.setUser(this);
        }
        reviews.add(review);
    }


    @Override
    public int hashCode() {
        return Objects.hash(id, login, password);
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null || getClass() != obj.getClass())
            return false;
        final UserEntity other = (UserEntity) obj;
        return Objects.equals(other.getId(), id) && Objects.equals(other.getLogin(), login)
                && Objects.equals(other.getPassword(), password);
    }
}
