package ip_2.lab1.users.service;

import ip_2.lab1.core.error.NotFoundException;
import ip_2.lab1.reviews.model.ReviewEntity;
import ip_2.lab1.reviews.repository.ReviewRepository;
import ip_2.lab1.users.model.UserEntity;
import ip_2.lab1.users.repository.UserRepository;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.StreamSupport;

@Service
public class UserService {
    private final UserRepository repository;
    private final ReviewRepository reviewRepository;

    public UserService(UserRepository repository, ReviewRepository reviewRepository){
        this.repository = repository;
        this.reviewRepository = reviewRepository;
    }

    private void checkLogin(String login) {
        if (repository.findByLoginIgnoreCase(login).isPresent()) {
            throw new IllegalArgumentException(
                    String.format("User with login %s is already exists", login));
        }
    }

    @Transactional(readOnly = true)
    public List<UserEntity> getAll(){
        return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
    }

    @Transactional(readOnly = true)
    public UserEntity get(Long id){
        return repository.findById(id)
                .orElseThrow(() -> new NotFoundException(UserEntity.class, id));
    }

    @Transactional(readOnly = true)
    public UserEntity getForReviewCreate(Long id){
        return repository.findByIdForReviewCreate(id)
                .orElseThrow(() -> new NotFoundException(UserEntity.class, id));
    }

    @Transactional
    public UserEntity create(UserEntity entity){
        if (entity == null){
            throw new IllegalArgumentException("Entity is null");
        }
        checkLogin(entity.getLogin());
        return repository.save(entity);
    }

    @Transactional
    public UserEntity update(Long id, UserEntity entity){
        if (entity == null)
            throw new IllegalArgumentException("Entity is null");
        final UserEntity existsEntity = get(id);
        existsEntity.setLogin(entity.getLogin());
        return repository.save(existsEntity);
    }

    @Transactional
    public UserEntity updatePassword(Long id, String password){
        if (password == null)
            throw new IllegalArgumentException("Password is null");
        final UserEntity existsEntity = get(id);
        if (password.equals(existsEntity.getPassword()))
            throw new IllegalArgumentException("It is existing password");
        if (password.contains("1234"))
            throw new IllegalArgumentException("Password could not contains numeric sequences");
        if(password.length() > 30 || password.length() < 4)
            throw new IllegalArgumentException("Password`s length must be in range 4 and 30 characters");
        existsEntity.setPassword(password);
        return repository.save(existsEntity);
    }

    @Transactional
    public UserEntity delete(Long id){
        final UserEntity existsEntity = get(id);
        reviewRepository.deleteAllByUserId(id);
        repository.deleteById(id);
        return existsEntity;
    }
}
