package ip_2.lab1;

import ip_2.lab1.core.error.NotFoundException;
import ip_2.lab1.items.model.ItemEntity;
import ip_2.lab1.items.service.ItemService;
import ip_2.lab1.types.model.TypeEntity;
import ip_2.lab1.types.service.TypeService;
import org.junit.jupiter.api.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.dao.DataIntegrityViolationException;

@SpringBootTest
@TestMethodOrder(MethodOrderer.OrderAnnotation.class)
public class ItemServiceTest {
    @Autowired
    private ItemService itemService;

    @Autowired
    private TypeService typeService;

    private TypeEntity type;
    private ItemEntity item;
    private ItemEntity item2;
    private ItemEntity item3;

    @BeforeEach
    void createData(){
        removeData();

        type = typeService.create(new TypeEntity("Asfalt", 1000));
        item = itemService.create(new ItemEntity(type, 900, 10));
        item2 = itemService.create(new ItemEntity(type, 800, 5));
        item3 = itemService.create(new ItemEntity(type, 700, 100));
    }

    @AfterEach
    void removeData(){
        itemService.getAll(0L).forEach(item -> itemService.delete(item.getId()));
        typeService.getAll().forEach(type -> typeService.delete(type.getId()));
    }


    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> itemService.get(0L));
    }

    @Test
    void createTest() {
        Assertions.assertEquals(3, itemService.getAll(0L).size());
        Assertions.assertEquals(item, itemService.get(item.getId()));
    }

    @Test
    void createNullableTest() {
        final ItemEntity NullableItem = new ItemEntity(null, null, null);
        Assertions.assertThrows(DataIntegrityViolationException.class, () -> itemService.create(NullableItem));
    }

    @Test
    void updateTest() {
        final int testPrice = 1800;
        final int testCount = 20;
        final TypeEntity type1 = typeService.create(new TypeEntity("Grunt", 2000));
        final TypeEntity oldType = item.getType();
        final int oldPrice = item.getPrice();
        final int oldCount = item.getCount();
        final ItemEntity newEntity = itemService.update(item.getId(), new ItemEntity(type1, testPrice, testCount));
        Assertions.assertEquals(3, itemService.getAll(0L).size());
        Assertions.assertEquals(newEntity, itemService.get(item.getId()));
        Assertions.assertEquals(type1, newEntity.getType());
        Assertions.assertEquals(testPrice, newEntity.getPrice());
        Assertions.assertEquals(testCount, newEntity.getCount());
        Assertions.assertNotEquals(oldType, newEntity.getType());
        Assertions.assertNotEquals(oldPrice, newEntity.getPrice());
        Assertions.assertNotEquals(oldCount, newEntity.getCount());
    }

    @Test
    void deleteTest() {
        itemService.delete(item3.getId());
        Assertions.assertEquals(2, itemService.getAll(0L).size());
        final ItemEntity last = itemService.get(item2.getId());
        Assertions.assertEquals(item2.getId(), last.getId());
    }
}

