package ip_2.lab1;

import ip_2.lab1.core.error.NotFoundException;
import ip_2.lab1.reviews.model.ReviewEntity;
import ip_2.lab1.reviews.service.ReviewService;
import ip_2.lab1.users.model.UserEntity;
import ip_2.lab1.users.service.UserService;
import org.junit.jupiter.api.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

@SpringBootTest
@TestMethodOrder(MethodOrderer.OrderAnnotation.class)
public class ReviewServiceTest {
    @Autowired
    private ReviewService reviewService;
    @Autowired
    private UserService userService;

    private UserEntity user;
    private ReviewEntity review;
    private ReviewEntity review2;
    private ReviewEntity review3;

    @BeforeEach
    void createData(){
        removeData();

        user = userService.create(new UserEntity("Artyom", "1234"));
        review = reviewService.create(new ReviewEntity(user, "Good"));
        review2 = reviewService.create(new ReviewEntity(user, "Super good"));
        review3 = reviewService.create(new ReviewEntity(user, "Not bad"));
    }

    @AfterEach
    void removeData(){
        reviewService.getAll(0L).forEach(review -> reviewService.delete(review.getId()));
        userService.getAll().forEach(user -> userService.delete(user.getId()));
    }

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> reviewService.get(0L));
    }

    @Test
    void createTest() {
        Assertions.assertEquals(3, reviewService.getAll(0L).size());
        Assertions.assertEquals(review, reviewService.get(review.getId()));
    }

    @Test
    void updateTest() {
        final String text = "test";
        final UserEntity user1 = userService.create(new UserEntity("Niyaz", "1234"));
        final UserEntity oldUser = review.getUser();
        final String oldText = review.getText();
        final ReviewEntity newEntity = reviewService.update(review.getId(), new ReviewEntity(user1, text));
        Assertions.assertEquals(3, reviewService.getAll(0L).size());
        Assertions.assertEquals(newEntity, reviewService.get(review.getId()));
        Assertions.assertEquals(user1, newEntity.getUser());
        Assertions.assertEquals(text, newEntity.getText());
        Assertions.assertNotEquals(oldUser, newEntity.getUser());
        Assertions.assertNotEquals(oldText, newEntity.getText());
    }

    @Test
    void deleteTest() {
        reviewService.delete(review3.getId());
        Assertions.assertEquals(2, reviewService.getAll(0L).size());
        final ReviewEntity last = reviewService.get(review2.getId());
        Assertions.assertEquals(review2.getId(), last.getId());
    }
}
