package ip_2.lab1;

import ip_2.lab1.types.service.TypeService;
import ip_2.lab1.core.error.NotFoundException;
import ip_2.lab1.types.model.TypeEntity;
import org.junit.jupiter.api.*;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.dao.DataIntegrityViolationException;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class TypeServiceTests {
	@Autowired
	private TypeService typeService;

	private TypeEntity type;

	@BeforeEach
	void createData() {
		removeData();

		type = typeService.create(new TypeEntity("Asfalt", 2000));
		typeService.create(new TypeEntity("Grunt", 3000));
		typeService.create(new TypeEntity("Asfalt + Grunt", 3500));
	}

	@AfterEach
	void removeData() {
		typeService.getAll().forEach(item -> typeService.delete(item.getId()));
	}

	@Test
	void getTest() {
		Assertions.assertThrows(NotFoundException.class, () -> typeService.get(0L));
	}

	@Test
	void createTest() {
		Assertions.assertEquals(3, typeService.getAll().size());
		Assertions.assertEquals(type, typeService.get(type.getId()));
	}

	@Test
	void createNotUniqueTest() {
		final TypeEntity nonUniqueType = new TypeEntity("Asfalt", 2000);
		Assertions.assertThrows(IllegalArgumentException.class, () -> typeService.create(nonUniqueType));
	}

	@Test
	void createNullableTest() {
		final TypeEntity nullableType = new TypeEntity(null, null);
		Assertions.assertThrows(DataIntegrityViolationException.class, () -> typeService.create(nullableType));
	}

	@Test
	void updateTest() {
		final String test = "TEST";
		final Integer testPrice = 1000;
		final String oldName = type.getName();
		final Integer oldPrice = type.getBasePrice();
		final TypeEntity newEntity = typeService.update(type.getId(), new TypeEntity(test, testPrice));
		Assertions.assertEquals(3, typeService.getAll().size());
		Assertions.assertEquals(newEntity, typeService.get(type.getId()));
		Assertions.assertEquals(test, newEntity.getName());
		Assertions.assertEquals(testPrice, newEntity.getBasePrice());
		Assertions.assertNotEquals(oldName, newEntity.getName());
		Assertions.assertNotEquals(oldPrice, newEntity.getBasePrice());
	}

	@Test
	void deleteTest() {
		typeService.delete(type.getId());
		Assertions.assertEquals(2, typeService.getAll().size());

		final TypeEntity newEntity = typeService.create(new TypeEntity(type.getName(), type.getBasePrice()));
		Assertions.assertEquals(3, typeService.getAll().size());
		Assertions.assertNotEquals(type.getId(), newEntity.getId());
	}
}
