package ip_2.lab1;

import ip_2.lab1.core.error.NotFoundException;
import ip_2.lab1.users.model.UserEntity;
import ip_2.lab1.users.service.UserService;
import org.junit.jupiter.api.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.dao.DataIntegrityViolationException;

@SpringBootTest
@TestMethodOrder(MethodOrderer.OrderAnnotation.class)
public class UserServiceTest {
    @Autowired
    private UserService userService;

    private UserEntity entity;

    @BeforeEach
    void createData() {
        removeData();

        entity = userService.create(new UserEntity("Artyom", "1234"));
        userService.create(new UserEntity("Niyaz", "1234"));
        userService.create(new UserEntity("Rostik", "1234"));
    }

    @AfterEach
    void removeData() {
        userService.getAll().forEach(item -> userService.delete(item.getId()));
    }

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> userService.get(0L));
    }

    @Test
    void createTest() {
        Assertions.assertEquals(3, userService.getAll().size());
        Assertions.assertEquals(entity, userService.get(entity.getId()));
    }

    @Test
    void createNotUniqueTest() {
        final UserEntity nonUniqueEntity = new UserEntity("Artyom", "1234");
        Assertions.assertThrows(IllegalArgumentException.class, () -> userService.create(nonUniqueEntity));
    }

    @Test
    void createNullableTest() {
        final UserEntity nullableEntity = new UserEntity(null, null);
        Assertions.assertThrows(DataIntegrityViolationException.class, () -> userService.create(nullableEntity));
    }

    @Test
    void updateTest() {
        final String login = "TEST";
        final String oldLogin = entity.getLogin();
        final String oldPassword = entity.getPassword();
        final UserEntity newEntity = userService.update(entity.getId(), new UserEntity(login, null));
        Assertions.assertEquals(3, userService.getAll().size());
        Assertions.assertEquals(newEntity, userService.get(entity.getId()));
        Assertions.assertEquals(login, newEntity.getLogin());
        Assertions.assertNotEquals(oldLogin, newEntity.getLogin());
    }

    @Test
    void updatePasswordTest(){
        final String password = "TEST";
        final String oldPassword = entity.getPassword();
        final UserEntity newEntity = userService.updatePassword(entity.getId(), password);
        Assertions.assertEquals(newEntity, userService.get(entity.getId()));
        Assertions.assertEquals(password, newEntity.getPassword());
        Assertions.assertNotEquals(oldPassword, newEntity.getPassword());
    }

    @Test
    void deleteTest() {
        userService.delete(entity.getId());
        Assertions.assertEquals(2, userService.getAll().size());

        final UserEntity newEntity = userService.create(new UserEntity(entity.getLogin(), entity.getPassword()));
        Assertions.assertEquals(3, userService.getAll().size());
        Assertions.assertNotEquals(entity.getId(), newEntity.getId());
    }
}
